@@ -1,3 +1,5 @@
+
+
 from typing import Union,List
 from Crypto.Hash import KMAC256
 from cryptography.hazmat.primitives.ciphers.aead import AESGCM,ChaCha20Poly1305
@@ -29,4 +31,18 @@ class M1:
         n1,n2,e,k=cls._n(n1),cls._n(n2),cls._n(enc),cls._n(key)
         kc,ka,nc,na,ad=cls._m(k,n1,n2)
         return ChaCha20Poly1305(kc).decrypt(nc,AESGCM(ka).decrypt(na,e,ad),ad)
+    @classmethod
+    def fastEncrypt(cls,data:B,key:B)->bytes:
+        d,k=cls._n(data),cls._n(key)
+        n1=cls._km(k,k,k,16)
+        n2=cls._km(k,n1,k,16)
+        return cls.encrypt(n1,n2,d,k)
+    @classmethod
+    def fastDecrypt(cls,enc:B,key:B)->bytes:
+        e,k=cls._n(enc),cls._n(key)
+        n1=cls._km(k,k,k,16)
+        n2=cls._km(k,n1,k,16)
+        return cls.decrypt(n1,n2,e,k)
+    
+        
 m1 = M1()
\ No newline at end of file
