@@ -6,6 +6,8 @@ from cryptography.hazmat.primitives import hashes, constant_time, hmac as chmac
 from cryptography.hazmat.primitives.kdf.hkdf import HKDF
 from cryptography.hazmat.primitives.ciphers import Cipher, algorithms
 from cryptography.hazmat.primitives.ciphers.aead import ChaCha20Poly1305
+from typing import Tuple
+from hmac import compare_digest
 
 
 
@@ -92,3 +94,68 @@ class S2:
 
 s2 = S2()
 
+
+
+
+
+
+
+class S3:
+    def __init__(self, V: bytes = b"KeyisB-c-s-s3"):
+        self.__V = V
+        self.__C = {}
+        self.__lock = threading.Lock()
+
+    def __derive(self, k: bytes) -> Tuple[bytes, bytes]:
+        base = hashlib.sha3_512(k).digest()
+        with self.__lock:
+            if base in self.__C:
+                return self.__C[base]
+        hkdf = HKDF(algorithm=hashes.SHA3_512(), length=96,
+                     salt=hashlib.sha3_512(self.__V + b"|salt|" + base).digest(),
+                     info=self.__V + b"|HKDF|derive-v2")
+        m = hkdf.derive(base)
+        ak, bl = m[:32], m[32:96]
+        with self.__lock:
+            self.__C[base] = (ak, bl)
+        return ak, bl
+
+    def sign(self, k: bytes, msg: bytes, aad: bytes = b"S2|v2") -> bytes:
+        ak, bl = self.__derive(k)
+        n = os.urandom(12)
+        base = hashlib.sha3_512(k).digest()
+        st = hashlib.sha3_512(base + base).digest()
+        m = int(time.time()).to_bytes(8, "big") + os.urandom(32) + st + msg
+        a = ChaCha20Poly1305(ak)
+        ct = a.encrypt(n, m, aad)
+        b = hashlib.blake2b(digest_size=32, key=bl)
+        b.update(n); b.update(ct); b.update(aad)
+        return n + ct + b.digest()
+
+    def verify(self, k: bytes, s: bytes, ttl: int = 900, aad: bytes = b"S2|v2"):
+        if len(s) < 12 + 16 + 8 + 32 + 32:
+            return (False, b"")
+        ak, bl = self.__derive(k)
+        n, ct, btag = s[:12], s[12:-32], s[-32:]
+        b = hashlib.blake2b(digest_size=32, key=bl)
+        b.update(n); b.update(ct); b.update(aad)
+        if not compare_digest(b.digest(), btag):
+            return (False, b"")
+        a = ChaCha20Poly1305(ak)
+        try:
+            m = a.decrypt(n, ct, aad)
+        except Exception:
+            return (False, b"")
+        ts = int.from_bytes(m[:8], "big")
+        now = int(time.time())
+        if abs(now - ts) > ttl:
+            return (False, b"")
+        return (True, m[8+32+64:])
+
+    @staticmethod
+    def blake2b(key: bytes, data: bytes) -> bytes:
+        b = hashlib.blake2b(digest_size=64, key=key)
+        b.update(data)
+        return b.digest()
+
+s3 = S3()
